import {audioDir, desktopDir, documentDir, downloadDir, homeDir, pictureDir, videoDir} from "./path.js";

/**
 * @type {Promise<Favourite[]>}
 */
const getDefaultFavourites = async () => {
    return [
        {
            name: 'Music',
            path: await audioDir(),
            system: true
        },
        {
            name: 'Documents',
            path: await documentDir(),
            system: true
        },
        {
            name: 'Desktop',
            path: await desktopDir(),
            system: true
        },
        {
            name: 'Downloads',
            path: await downloadDir(),
            system: true
        },
        {
            name: 'Home',
            path: await homeDir(),
            system: true
        },
        {
            name: 'Pictures',
            path: await pictureDir(),
            system: true
        },
        {
            name: 'Videos',
            path: await videoDir(),
            system: true
        },
    ]
}

const defaultConfig = JSON.stringify({
    favourites: await getDefaultFavourites()
});

let config = null;
const persist = () => localStorage.setItem('app:config', JSON.stringify(config));
const load = () => config = JSON.parse(localStorage.getItem('app:config') ?? defaultConfig);
const ensureConfig = () => config ? '' : load();

/**
 * @param {string} key
 * @param {any} value
 */
export const setConfig = (key, value) => {
    config[key] = value;

    persist();

    return value;
};

/**
 *
 * @param {string} key
 * @param {any} fallback
 * @returns {any}
 */
export const getConfig =  (key, fallback) => {
    ensureConfig();

    if (key in config) {
        return config[key];
    }

    return fallback;
};

/**
 * @typedef {Object} Favourite
 * @property {string} name
 * @property {string} path
 * @property {number} index
 */

/**
 * @param favourites
 * @returns {Favourite[]}
 */
export const setFavourites = (favourites) => setConfig('favourites', favourites);

/**
 * @returns {Favourite[]}
 */
export const getFavourites = () => getConfig('favourites', []);

/**
 * @param {Favourite} favourite
 *
 * @return {Favourite[]}
 */
export const addFavourite = (favourite) => {
    const favourites = getFavourites();

    favourites.push(favourite);

    setFavourites(favourites);

    return favourites;
}

export const removeFavouriteByIndex = (index) => {
    const favourites = getFavourites();

    favourites.splice(index, 1)

    setFavourites(favourites);

    return favourites;
}